@@ -3,7 +3,7 @@
 # vim:fenc=utf-8
 
 """
-Copyright 2023 Bennett Meares
+Copyright 2025 Bennett Meares
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -18,24 +18,46 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 
+import atexit
+
+from meerschaum.utils.typing import SuccessTuple
+from meerschaum.utils.packages import attempt_import
 from meerschaum.core.Pipe import Pipe
 from meerschaum.plugins import Plugin
-from meerschaum.utils import get_pipes
 from meerschaum.utils.venv import Venv
+from meerschaum.jobs import Job, make_executor
+from meerschaum.connectors import get_connector, Connector, InstanceConnector, make_connector
+from meerschaum.utils import get_pipes
 from meerschaum.utils.formatting import pprint
 from meerschaum._internal.docs import index as __doc__
-from meerschaum.connectors import get_connector
 from meerschaum.config import __version__, get_config
-from meerschaum.utils.packages import attempt_import
+from meerschaum._internal.entry import entry
+from meerschaum.__main__ import _close_pools
 
-__pdoc__ = {'gui': False, 'api': False, 'core': False,}
+atexit.register(_close_pools)
+
+__pdoc__ = {'gui': False, 'api': False, 'core': False, '_internal': False}
 __all__ = (
-    "Pipe",
     "get_pipes",
     "get_connector",
+    "get_config",
+    "Pipe",
     "Plugin",
+    "SuccessTuple",
     "Venv",
     "Plugin",
+    "Job",
     "pprint",
     "attempt_import",
+    "actions",
+    "config",
+    "connectors",
+    "jobs",
+    "plugins",
+    "utils",
+    "SuccessTuple",
+    "Connector",
+    "InstanceConnector",
+    "make_connector",
+    "entry",
 )
