@@ -9,59 +9,21 @@ Default actions available to the mrsm CLI.
 from __future__ import annotations
 from meerschaum.utils.typing import Callable, Any, Optional, Union, List, Dict, SuccessTuple
 from meerschaum.utils.packages import get_modules_from_package
-from meerschaum.utils.warnings import enable_depreciation_warnings
-enable_depreciation_warnings(__name__)
-_custom_actions = []
 
-### build __all__ from other .py files in this package
-import sys
-modules = get_modules_from_package(
-    sys.modules[__name__],
-    names = False,
+__all__ = (
+    'get_action',
+    'get_subactions',
+    'make_action',
+    'pre_sync_hook',
+    'post_sync_hook',
+    'get_main_action_name',
+    'get_completer',
 )
-__all__ = ['actions', 'get_subactions', 'get_action', 'get_main_action_name', 'get_completer']
-
-### Build the actions dictionary by importing all
-### functions that do not begin with '_' from all submodules.
-from inspect import getmembers, isfunction
-actions = {}
-"This docstring will be replaced in __pdoc__ at the end of this file."
-
-for module in modules:
-    ### A couple important things happening here:
-    ### 1. Find all functions in all modules in `actions` package
-    ###     (skip functions that begin with '_')
-    ### 2. Add them as members to the Shell class
-    ###     - Original definition : meerschaum._internal.shell.Shell
-    ###     - New definition      : meerschaum._internal.Shell
-    ### 3. Populate the actions dictionary with function names and functions
-    ###
-    ### UPDATE:
-    ### Shell modifications have been deferred to get_shell in order to improve lazy loading.
-
-    actions.update(
-        dict(
-            [
-                ### __name__ and new function pointer
-                (ob[0], ob[1])
-                    for ob in getmembers(module)
-                        if isfunction(ob[1])
-                            ### check that the function belongs to the module
-                            and ob[0] == module.__name__.replace('_', '').split('.')[-1]
-                            ### skip functions that start with '_'
-                            and ob[0][0] != '_'
-            ]
-        )
-    )
-
-original_actions = actions.copy()
-from meerschaum._internal.entry import entry
-
 
 def get_subactions(
-        action: Union[str, List[str]],
-        _actions: Optional[Dict[str, Callable[[Any], Any]]] = None,
-    ) -> Dict[str, Callable[[Any], Any]]:
+    action: Union[str, List[str]],
+    _actions: Optional[Dict[str, Callable[[Any], Any]]] = None,
+) -> Dict[str, Callable[[Any], Any]]:
     """
     Return a dictionary of an action's sub-action functions.
 
@@ -87,7 +49,11 @@ def get_subactions(
     for name, f in inspect.getmembers(action_module):
         if not inspect.isfunction(f):
             continue
-        if action_function.__name__ + '_' in name and not name.lstrip('_').startswith('complete'):
+
+        ### Detect subactions which may contain an underscore prefix.
+        if (
+            name.lstrip('_').startswith(action_function.__name__.lstrip('_') + '_')
+        ):
             _name = name.replace(action_function.__name__, '')
             _name = _name.lstrip('_')
             subactions[_name] = f
@@ -95,9 +61,9 @@ def get_subactions(
 
 
 def get_action(
-        action: Union[str, List[str]],
-        _actions: Optional[Dict[str, Callable[[Any], Any]]] = None,
-    ) -> Union[Callable[[Any], Any], None]:
+    action: Union[str, List[str]],
+    _actions: Optional[Dict[str, Callable[[Any], Any]]] = None,
+) -> Union[Callable[[Any], Any], None]:
     """
     Return a function corresponding to the given action list.
     This may be a custom action with an underscore, in which case, allow for underscores.
@@ -135,6 +101,8 @@ def get_action(
     if action[0] in _actions:
         subactions = get_subactions([action[0]], _actions=_actions)
         if action[1] not in subactions:
+            if (action[1] + 's') in subactions:
+                return subactions[action[1] + 's']
             return _actions[action[0]]
         return subactions[action[1]]
 
@@ -142,9 +110,9 @@ def get_action(
 
 
 def get_main_action_name(
-        action: Union[List[str], str],
-        _actions: Optional[Dict[str, Callable[[Any], SuccessTuple]]] = None,
-    ) -> Union[str, None]:
+    action: Union[List[str], str],
+    _actions: Optional[Dict[str, Callable[[Any], SuccessTuple]]] = None,
+) -> Union[str, None]:
     """
     Given an action list, return the name of the main function.
     For subactions, this will return the root function.
@@ -181,9 +149,9 @@ def get_main_action_name(
 
 
 def get_completer(
-        action: Union[List[str], str],
-        _actions: Optional[Dict[str, Callable[[Any], SuccessTuple]]] = None,
-    ) -> Union[
+    action: Union[List[str], str],
+    _actions: Optional[Dict[str, Callable[[Any], SuccessTuple]]] = None,
+) -> Union[
         Callable[['meerschaum._internal.shell.Shell', str, str, int, int], List[str]], None
     ]:
     """Search for a custom completer function for an action."""
@@ -219,14 +187,167 @@ def get_completer(
     return None
 
 
-from meerschaum._internal.entry import get_shell
+def choose_subaction(
+    action: Optional[List[str]] = None,
+    options: Optional[Dict[str, Any]] = None,
+    **kw
+) -> SuccessTuple:
+    """
+    Given a dictionary of options and the standard Meerschaum actions list,
+    check if choice is valid and execute chosen function, else show available
+    options and return False
+
+    Parameters
+    ----------
+    action: Optional[List[str]], default None
+        A list of subactions (e.g. `show pipes` -> ['pipes']).
+
+    options: Optional[Dict[str, Any]], default None
+        Available options to execute.
+        option (key) -> function (value)
+        Functions must accept **kw keyword arguments
+        and return a tuple of success bool and message.
+
+    Returns
+    -------
+    The return value of the chosen subaction (assumed to be a `SuccessTuple`).
+
+    """
+    from meerschaum.utils.warnings import warn, info
+    import inspect
+    if action is None:
+        action = []
+    if options is None:
+        options = {}
+    parent_action = inspect.stack()[1][3]
+    if len(action) == 0:
+        action = ['']
+    choice = action[0]
+
+    def valid_choice(_choice : str, _options : dict):
+        if _choice in _options:
+            return _choice
+        if (_choice + 's') in options:
+            return _choice + 's'
+        return None
+
+    parsed_choice = valid_choice(choice, options)
+    if parsed_choice is None:
+        warn(f"Cannot {parent_action} '{choice}'. Choose one:", stack=False)
+        for option in sorted(options):
+            print(f"  - {parent_action} {option}")
+        return (False, f"Invalid choice '{choice}'.")
+    ### remove parent sub-action
+    kw['action'] = list(action)
+    del kw['action'][0]
+    return options[parsed_choice](**kw)
+
+
+def _get_subaction_names(action: str, globs: dict = None) -> List[str]:
+    """Use `meerschaum.actions.get_subactions()` instead."""
+    if globs is None:
+        import importlib
+        module = importlib.import_module(f'meerschaum.actions.{action}')
+        globs = vars(module)
+    subactions = []
+    for item in globs:
+        if f'_{action}' in item and 'complete' not in item.lstrip('_'):
+            subactions.append(globs[item])
+    return subactions
+
+
+def choices_docstring(action: str, globs: Optional[Dict[str, Any]] = None) -> str:
+    """
+    Append the an action's available options to the module docstring.
+    This function is to be placed at the bottom of each action module.
+
+    Parameters
+    ----------
+    action: str
+        The name of the action module (e.g. 'install').
+        
+    globs: Optional[Dict[str, Any]], default None
+        An optional dictionary of global variables.
+
+    Returns
+    -------
+    The generated docstring for the module.
+
+    Examples
+    --------
+    >>> from meerschaum.utils.misc import choices_docstring as _choices_docstring
+    >>> install.__doc__ += _choices_docstring('install')
+
+    """
+    options_str = f"\n    Options:\n        `{action} "
+    subactions = _get_subaction_names(action, globs=globs)
+    options_str += "["
+    sa_names = []
+    for sa in subactions:
+        try:
+            sa_names.append(sa.__name__[len(f"_{action}") + 1:])
+        except Exception as e:
+            print(e)
+            return ""
+    for sa_name in sorted(sa_names):
+        options_str += f"{sa_name}, "
+    options_str = options_str[:-2] + "]`"
+    return options_str
+
+### build __all__ from other .py files in this package
+import sys
+modules = get_modules_from_package(
+    sys.modules[__name__],
+    names = False,
+)
+__all__ = ['actions', 'get_subactions', 'get_action', 'get_main_action_name', 'get_completer']
+
+### Build the actions dictionary by importing all
+### functions that do not begin with '_' from all submodules.
+from inspect import getmembers, isfunction
+actions = {}
+_custom_actions_plugins: Dict[str, str] = {}
+_plugins_actions: Dict[str, List[str]] = {}
+
+for module in modules:
+    ### A couple important things happening here:
+    ### 1. Find all functions in all modules in `actions` package
+    ###     (skip functions that begin with '_')
+    ### 2. Add them as members to the Shell class
+    ###     - Original definition : meerschaum._internal.shell.Shell
+    ###     - New definition      : meerschaum._internal.Shell
+    ### 3. Populate the actions dictionary with function names and functions
+    ###
+    ### UPDATE:
+    ### Shell modifications have been deferred to get_shell in order to improve lazy loading.
+
+    actions.update(
+        dict(
+            [
+                ### __name__ and new function pointer
+                (ob[0], ob[1])
+                    for ob in getmembers(module)
+                        if isfunction(ob[1])
+                            ### check that the function belongs to the module
+                            and ob[0] == module.__name__.replace('_', '').split('.')[-1]
+                            ### skip functions that start with '__'
+                            and ob[0][0] != '__'
+            ]
+        )
+    )
+
+original_actions = actions.copy()
+from meerschaum._internal.entry import entry, get_shell
 import meerschaum.plugins
-#  meerschaum.plugins.load_plugins()
 make_action = meerschaum.plugins.make_action
+pre_sync_hook = meerschaum.plugins.pre_sync_hook
+post_sync_hook = meerschaum.plugins.post_sync_hook
 
 ### Instruct pdoc to skip the `meerschaum.actions.plugins` subdirectory.
 __pdoc__ = {
-    'plugins' : False, 'arguments': True, 'shell': False,
+    'plugins': False,
+    'arguments': True,
+    'shell': False,
     'actions': (
         "Access functions of the standard Meerschaum actions.\n\n"
         + "Visit the [actions reference page](https://meerschaum.io/reference/actions/) "
@@ -242,4 +363,5 @@ __pdoc__ = {
 }
 for a in actions:
     __pdoc__[a] = False
-meerschaum.plugins.load_plugins()
+
+meerschaum.plugins.load_plugins(skip_if_loaded=True)
