@@ -8,18 +8,20 @@ Callbacks for the registration page.
 
 import uuid
 from meerschaum.api import get_api_connector, endpoints, CHECK_UPDATE
-from meerschaum.api.dash import dash_app, debug, active_sessions
+from meerschaum.api.dash import dash_app, debug
+from meerschaum.api.dash.sessions import set_session
 from dash.dependencies import Input, Output, State, ALL, MATCH
 from dash.exceptions import PreventUpdate
 from meerschaum.core import User
-from meerschaum.config.static import STATIC_CONFIG
+from meerschaum._internal.static import STATIC_CONFIG
 from meerschaum.utils.packages import attempt_import
+from meerschaum.api._oauth2 import CustomOAuth2PasswordRequestForm
 dash = attempt_import('dash', check_update=CHECK_UPDATE)
 from fastapi.exceptions import HTTPException
 
 @dash_app.callback(
-    [Output("username-input", "valid"), Output("username-input", "invalid")],
-    [Input("username-input", "value")],
+    [Output("register-username-input", "valid"), Output("register-username-input", "invalid")],
+    [Input("register-username-input", "value")],
 )
 def validate_username(username):
     if not username:
@@ -34,8 +36,8 @@ def validate_username(username):
     return valid, not valid
 
 @dash_app.callback(
-    [Output("password-input", "valid"), Output("password-input", "invalid")],
-    [Input("password-input", "value")],
+    [Output("register-password-input", "valid"), Output("register-password-input", "invalid")],
+    [Input("register-password-input", "value")],
 )
 def validate_password(password):
     if not password:
@@ -44,8 +46,8 @@ def validate_password(password):
     return valid, not valid
 
 @dash_app.callback(
-    [Output("email-input", "valid"), Output("email-input", "invalid")],
-    [Input("email-input", "value")],
+    [Output("register-email-input", "valid"), Output("register-email-input", "invalid")],
+    [Input("register-email-input", "value")],
 )
 def validate_email(email):
     if not email:
@@ -56,8 +58,8 @@ def validate_email(email):
 
 @dash_app.callback(
     Output('session-store', 'data'),
-    Output('username-input', 'className'),
-    Output('location', 'pathname'),
+    Output('register-username-input', 'className'),
+    Output('mrsm-location', 'pathname'),
     Input('register-username-input', 'n_submit'),
     Input('register-password-input', 'n_submit'),
     Input('register-button', 'n_clicks'),
@@ -66,10 +68,16 @@ def validate_email(email):
     State("register-email-input", "value"),
 )
 def register_button_click(
-        username_submit,
-        password_submit,
-        n_clicks, username, password, email
-    ):
+    username_submit,
+    password_submit,
+    n_clicks,
+    username,
+    password,
+    email,
+):
+    """
+    Register the user and redirect to the console.
+    """
     if not n_clicks:
         raise PreventUpdate
     form_class = 'form-control'
@@ -83,15 +91,22 @@ def register_button_click(
     user = User(username, password, email=email, instance=conn)
     user_id = conn.get_user_id(user, debug=debug)
     if user_id is not None:
+        form_class += ' is-invalid'
         return {}, form_class, dash.no_update
     success, msg = conn.register_user(user, debug=debug)
     if not success:
         form_class += ' is-invalid'
         return {}, form_class, dash.no_update
     try:
-        token_dict = login({'username' : username, 'password' : password})
+        form = CustomOAuth2PasswordRequestForm(
+            grant_type='password',
+            username=username,
+            password=password,
+            scope=' '.join(STATIC_CONFIG['tokens']['scopes'])
+        )
+        _ = login(form)
         session_data = {'session-id': str(uuid.uuid4())}
-        active_sessions[session_data['session-id']] = {'username': username}
+        set_session(session_data['session-id'], {'username': username})
     except HTTPException:
         form_class += ' is-invalid'
         session_data = None
