@@ -19,51 +19,51 @@ allow_user_registration = permissions_config['registration']['users']
 
 registration_div = html.Div(
     id = 'registration-div',
-    style = {'height' : '100%'},
-    children = (
+    style={'height' : '100%'},
+    children=(
         [
             dcc.Link(
                 'No account? Create one here.',
-                href = (dash_endpoint + '/register'),
-                refresh = False
+                href=(dash_endpoint + '/register'),
+                refresh=False
             ),
         ] if allow_user_registration
         else [
             dcc.Markdown("""
                 #### **Web registration is disabled for security.**
-                You can still register users on this instance with a SQL connector.
+                You can register users via the CLI with `mrsm register user`.
             """),
             dbc.Button(
                 'More information.',
                 id = 'show-user-registration-disabled-button',
-                color = 'link',
-                size = 'sm',
+                color='link',
+                size='sm',
             ),
             dbc.Collapse(
-                id = 'user-registration-disabled-collapse',
-                children = [
+                id='user-registration-disabled-collapse',
+                children=[
                     dcc.Markdown(
-                        "For example, to register user `newuser` on instance `sql:main`):"
+                        "For example, to register user `newuser` on instance `sql:main`:"
                     ),
                     html.Pre(
                         html.Code('mrsm register user newuser -i sql:main', className='codeblock'),
                     ),
                     dcc.Markdown("""
-                        To enable online registration, open the `system` configuration file and""" +
+                        To enable online registration, open the `api` configuration file and""" +
                     """ set the permissions to `true`:"""
                     ),
                     html.Pre(
-                        html.Code('mrsm edit config system', className='codeblock'),
+                        html.Code('mrsm edit config api', className='codeblock'),
                     ),
                     html.Br(),
                     dcc.Markdown('The settings file should look something like this:'),
                     html.Pre(
                         html.Code(
                             json.dumps({
-                                'api' : {
-                                    'permissions' : {
-                                        'registration' : {
-                                            'users' : True,
+                                'api': {
+                                    'permissions': {
+                                        'registration': {
+                                            'users': True,
                                         },
                                     }
                                 }
@@ -80,6 +80,7 @@ layout = dbc.Container([
     html.Br(),
     dbc.Container([
         dcc.Location(id='location-login', refresh=True),
+        html.Div(id="login-alert-div"),
         html.Div([
             dbc.Container(
                 html.Img(
@@ -111,11 +112,11 @@ layout = dbc.Container([
                 dbc.Row([
                     dbc.Col([
                         html.Button(
-                            children = 'Login',
-                            n_clicks = 0,
-                            type = 'submit',
-                            id = 'login-button',
-                            className = 'btn btn-primary btn-lg'
+                            children='Login',
+                            n_clicks=0,
+                            type='submit',
+                            id='login-button',
+                            className='btn btn-primary btn-lg'
                         ),
                     ]),
                 ]),
@@ -129,4 +130,3 @@ layout = dbc.Container([
         ]),
     ], className='jumbotron')
 ])
-
