@@ -7,15 +7,14 @@ Return version information
 """
 
 import fastapi
-from meerschaum.api import app, endpoints, private, manager
-from meerschaum.utils.typing import Union
+from meerschaum.api import app, endpoints, private, ScopedAuth
 
 @app.get(endpoints['version'], tags=['Version'])
 def get_api_version(
-        curr_user = (
-            fastapi.Depends(manager) if private else None
-        ),
-    ):
+    curr_user = (
+        fastapi.Depends(ScopedAuth(['instance:read'])) if private else None
+    ),
+):
     """
     Get the Meerschaum API version.
     """
@@ -24,13 +23,12 @@ def get_api_version(
 
 @app.get(endpoints['version'] + "/mrsm", tags=['Version'])
 def get_meerschaum_version(
-        curr_user = (
-            fastapi.Depends(manager) if private else None
-        ),
-    ):
+    curr_user = (
+        fastapi.Depends(ScopedAuth(['instance:read'])) if private else None
+    ),
+):
     """
     Get the Meerschaum instance version.
     """
     from meerschaum import __version__ as version
     return version
-
