@@ -13,66 +13,68 @@ For ease of use, you can also import from the root `meerschaum` module:
 
 from __future__ import annotations
 
-from meerschaum.utils.typing import Any, SuccessTuple, Union, Optional, List, Dict
-from meerschaum.utils.threading import Lock, RLock
-from meerschaum.utils.warnings import error, warn
+import meerschaum as mrsm
+from meerschaum.utils.typing import Any, Union, List, Dict, Optional
+from meerschaum.utils.threading import RLock
+from meerschaum.utils.warnings import warn
 
-from meerschaum.connectors.Connector import Connector, InvalidAttributesError
-from meerschaum.connectors.sql.SQLConnector import SQLConnector
-from meerschaum.connectors.api.APIConnector import APIConnector
-from meerschaum.connectors.sql._create_engine import flavor_configs as sql_flavor_configs
+from meerschaum.connectors._Connector import Connector, InvalidAttributesError
+from meerschaum.connectors.instance._InstanceConnector import InstanceConnector
+from meerschaum.connectors.sql._SQLConnector import SQLConnector
+from meerschaum.connectors.api._APIConnector import APIConnector
 
-__all__ = ("Connector", "SQLConnector", "APIConnector", "get_connector", "is_connected")
+__all__ = (
+    "make_connector",
+    "Connector",
+    "InstanceConnector",
+    "SQLConnector",
+    "APIConnector",
+    "get_connector",
+    "is_connected",
+    "poll",
+    "api",
+    "sql",
+    "valkey",
+    "parse",
+)
 
 ### store connectors partitioned by
 ### type, label for reuse
 connectors: Dict[str, Dict[str, Connector]] = {
-    'api'   : {},
-    'sql'   : {},
-    'plugin': {},
+    'api'    : {},
+    'sql'    : {},
+    'plugin' : {},
+    'valkey' : {},
 }
 instance_types: List[str] = ['sql', 'api']
 _locks: Dict[str, RLock] = {
     'connectors'               : RLock(),
     'types'                    : RLock(),
     'custom_types'             : RLock(),
+    'plugins_types'            : RLock(),
     '_loaded_plugin_connectors': RLock(),
     'instance_types'           : RLock(),
 }
-attributes: Dict[str, Dict[str, Any]] = {
-    'api': {
-        'required': [
-            'host',
-            'username',
-            'password'
-        ],
-        'default': {
-            'protocol': 'http',
-            'port'    : 8000,
-        },
-    },
-    'sql': {
-        'flavors': sql_flavor_configs,
-    },
-}
+
 ### Fill this with objects only when connectors are first referenced.
 types: Dict[str, Any] = {}
 custom_types: set = set()
+plugins_types: Dict[str, List[str]] = {}
+_known_custom_types: set = set()
 _loaded_plugin_connectors: bool = False
 
 
 def get_connector(
-        type: str = None,
-        label: str = None,
-        refresh: bool = False,
-        debug: bool = False,
-        **kw: Any
-    ) -> Connector:
+    type: str = None,
+    label: str = None,
+    refresh: bool = False,
+    debug: bool = False,
+    **kw: Any
+) -> Connector:
     """
     Return existing connector or create new connection and store for reuse.
     
     You can create new connectors if enough parameters are provided for the given type and flavor.
-    
 
     Parameters
     ----------
@@ -114,15 +116,18 @@ def get_connector(
     """
     from meerschaum.connectors.parse import parse_instance_keys
     from meerschaum.config import get_config
-    from meerschaum.config.static import STATIC_CONFIG
+    from meerschaum._internal.static import STATIC_CONFIG
     from meerschaum.utils.warnings import warn
     global _loaded_plugin_connectors
     if isinstance(type, str) and not label and ':' in type:
         type, label = type.split(':', maxsplit=1)
+
     with _locks['_loaded_plugin_connectors']:
         if not _loaded_plugin_connectors:
             load_plugin_connectors()
+            _load_builtin_custom_connectors()
             _loaded_plugin_connectors = True
+
     if type is None and label is None:
         default_instance_keys = get_config('meerschaum', 'instance', patch=True)
         ### recursive call to get_connector
@@ -153,13 +158,15 @@ def get_connector(
 
     if 'sql' not in types:
         from meerschaum.connectors.plugin import PluginConnector
+        from meerschaum.connectors.valkey import ValkeyConnector
         with _locks['types']:
             types.update({
-                'api'   : APIConnector,
-                'sql'   : SQLConnector,
+                'api': APIConnector,
+                'sql': SQLConnector,
                 'plugin': PluginConnector,
+                'valkey': ValkeyConnector,
             })
-    
+
     ### determine if we need to call the constructor
     if not refresh:
         ### see if any user-supplied arguments differ from the existing instance
@@ -249,11 +256,11 @@ def is_connected(keys: str, **kw) -> bool:
 
     try:
         typ, label = keys.split(':')
-    except Exception as e:
+    except Exception:
         return False
     if typ not in instance_types:
         return False
-    if not (label in connectors.get(typ, {})):
+    if label not in connectors.get(typ, {}):
         return False
 
     from meerschaum.connectors.parse import parse_instance_keys
@@ -262,13 +269,11 @@ def is_connected(keys: str, **kw) -> bool:
         with warnings.catch_warnings():
             warnings.filterwarnings('ignore')
             return conn.test_connection(**kw)
-    except Exception as e:
+    except Exception:
         return False
 
 
-def make_connector(
-        cls,
-    ):
+def make_connector(cls, _is_executor: bool = False):
     """
     Register a class as a `Connector`.
     The `type` will be the lower case of the class name, without the suffix `connector`.
@@ -283,21 +288,34 @@ def make_connector(
     --------
     >>> import meerschaum as mrsm
     >>> from meerschaum.connectors import make_connector, Connector
+    >>> 
+    >>> @make_connector
     >>> class FooConnector(Connector):
-    ...     def __init__(self, label: str, **kw):
-    ...         super().__init__('foo', label, **kw)
+    ...     REQUIRED_ATTRIBUTES: list[str] = ['username', 'password']
     ... 
-    >>> make_connector(FooConnector)
-    >>> mrsm.get_connector('foo', 'bar')
-    foo:bar
+    >>> conn = mrsm.get_connector('foo:bar', username='dog', password='cat')
+    >>> print(conn.username, conn.password)
+    dog cat
     >>> 
     """
     import re
-    typ = re.sub(r'connector$', '', cls.__name__.lower())
+    from meerschaum.plugins import _get_parent_plugin
+    suffix_regex = (
+        r'connector$'
+        if not _is_executor
+        else r'executor$'
+    )
+    plugin_name = _get_parent_plugin(2)
+    typ = re.sub(suffix_regex, '', cls.__name__.lower())
     with _locks['types']:
         types[typ] = cls
     with _locks['custom_types']:
         custom_types.add(typ)
+    if plugin_name:
+        with _locks['plugins_types']:
+            if plugin_name not in plugins_types:
+                plugins_types[plugin_name] = []
+            plugins_types[plugin_name].append(typ)
     with _locks['connectors']:
         if typ not in connectors:
             connectors[typ] = {}
@@ -317,18 +335,44 @@ def load_plugin_connectors():
     from meerschaum.plugins import get_plugins, import_plugins
     to_import = []
     for plugin in get_plugins():
+        if plugin is None:
+            continue
         with open(plugin.__file__, encoding='utf-8') as f:
             text = f.read()
-        if 'make_connector' in text:
+        if 'make_connector' in text or 'Connector' in text:
             to_import.append(plugin.name)
     if not to_import:
         return
-    import_plugins(*to_import) 
+    import_plugins(*to_import)
+
+
+def unload_plugin_connectors(
+    plugin_names: Optional[List[str]] = None,
+    debug: bool = False,
+) -> None:
+    """
+    Unload custom connectors added by plugins.
+    """
+    from meerschaum.plugins import get_plugins_names
+    global custom_types, _known_custom_types, types, plugins_types, connectors
+
+    plugin_names = plugin_names or get_plugins_names()
+
+    for plugin_name in plugin_names:
+        plugin_types = plugins_types.get(plugin_name, [])
+        for typ in plugin_types:
+            _ = types.pop(typ, None)
+            _ = connectors.pop(typ, None)
+            if typ in instance_types:
+                instance_types.remove(typ)
+
+    custom_types.clear()
+    custom_types.update(_known_custom_types)
 
 
 def get_connector_plugin(
-        connector: Connector,
-    ) -> Union[str, None, 'meerschaum.Plugin']:
+    connector: Connector,
+) -> Union[str, None, mrsm.Plugin]:
     """
     Determine the plugin for a connector.
     This is useful for handling virtual environments for custom instance connectors.
@@ -344,7 +388,6 @@ def get_connector_plugin(
     """
     if not hasattr(connector, 'type'):
         return None
-    from meerschaum import Plugin
     plugin_name = (
         connector.__module__.replace('plugins.', '').split('.')[0]
         if connector.type in custom_types else (
@@ -353,5 +396,15 @@ def get_connector_plugin(
             else 'mrsm'
         )
     )
-    plugin = Plugin(plugin_name)
+    plugin = mrsm.Plugin(plugin_name)
     return plugin if plugin.is_installed() else None
+
+
+def _load_builtin_custom_connectors():
+    """
+    Import custom connectors decorated with `@make_connector` or `@make_executor`.
+    """
+    import meerschaum.jobs.systemd
+    import meerschaum.connectors.valkey
+    _known_custom_types.add('valkey')
+    _known_custom_types.add('systemd')
