@@ -7,28 +7,31 @@ Utility functions for parsing connector keys.
 """
 
 from __future__ import annotations
-from meerschaum.utils.typing import Mapping, Any, SuccessTuple, Union, Optional, Dict, Tuple
+
+import meerschaum as mrsm
+from meerschaum.utils.typing import Any, Union, Optional, Dict, Tuple
+
 
 def parse_connector_keys(
-        keys: str,
-        construct: bool = True,
-        as_tuple: bool = False,
-        **kw:  Any
-    ) -> (
-        Union[
-            meerschaum.connectors.Connector,
-            Dict[str, Any],
-            Tuple[
-                Union[
-                    meerschaum.connectors.Connector,
-                    Dict[str, Any],
-                    None
-                ],
-                str
+    keys: str,
+    construct: bool = True,
+    as_tuple: bool = False,
+    **kw:  Any
+) -> (
+    Union[
+        mrsm.connectors.Connector,
+        Dict[str, Any],
+        Tuple[
+            Union[
+                mrsm.connectors.Connector,
+                Dict[str, Any],
+                None,
             ],
-            None
-        ]
-    ):
+            str
+        ],
+        None
+    ]
+):
     """
     Convenience function for parsing connector keys and returning Connector objects.
 
@@ -53,14 +56,14 @@ def parse_connector_keys(
     import copy
     from meerschaum.connectors import get_connector
     from meerschaum.config import get_config
-    from meerschaum.config.static import _static_config
+    from meerschaum._internal.static import STATIC_CONFIG
     from meerschaum.utils.warnings import error
 
     ### `get_connector()` handles the logic for falling back to 'main',
     ### so don't make any decisions here.
     vals = str(keys).split(':')
     _type = vals[0]
-    _label = vals[1] if len(vals) > 1 else _static_config()['connectors']['default_label']
+    _label = vals[1] if len(vals) > 1 else STATIC_CONFIG['connectors']['default_label']
     _get_connector_kw = {'type': _type, 'label': _label}
     _get_connector_kw.update(kw)
 
@@ -87,23 +90,20 @@ def parse_connector_keys(
 
 
 def parse_instance_keys(
-        keys: Optional[str],
-        construct: bool = True,
-        as_tuple: bool = False,
-        **kw
-    ):
+    keys: Optional[str],
+    construct: bool = True,
+    as_tuple: bool = False,
+    **kw
+):
     """
     Parse the Meerschaum instance value into a Connector object.
     """
-    from meerschaum.utils.warnings import warn
     from meerschaum.config import get_config
 
-    ### TODO Check for valid types? Not sure how to do that if construct = False.
-
     if keys is None:
         keys = get_config('meerschaum', 'instance')
     keys = str(keys)
-    
+
     return parse_connector_keys(keys, construct=construct, as_tuple=as_tuple, **kw)
 
 
@@ -111,22 +111,38 @@ def parse_repo_keys(keys: Optional[str] = None, **kw):
     """Parse the Meerschaum repository value into an APIConnector."""
     from meerschaum.config import get_config
     if keys is None:
-        keys = get_config('meerschaum', 'default_repository', patch=True)
+        keys = get_config('meerschaum', 'repository', patch=True)
     keys = str(keys)
     if ':' not in keys:
         keys = 'api:' + keys
 
+    if not keys.startswith('api:'):
+        raise ValueError("Only APIConnectors may be treated as repositories.")
+
+    return parse_connector_keys(keys, **kw)
+
+
+def parse_executor_keys(keys: Optional[str] = None, **kw):
+    """Parse the executor keys into an APIConnector or string."""
+    from meerschaum.jobs import get_executor_keys_from_context
+    if keys is None:
+        keys = get_executor_keys_from_context()
+
+    if keys is None or keys == 'local':
+        return 'local'
+
+    keys = str(keys)
     return parse_connector_keys(keys, **kw)
 
 
 def is_valid_connector_keys(
-        keys: str
-    ) -> bool:
-    """Verify a connector_keys string references a valid connector.
+    keys: str
+) -> bool:
+    """
+    Verify a connector_keys string references a valid connector.
     """
     try:
         success = parse_connector_keys(keys, construct=False) is not None
-    except Exception as e:
+    except Exception:
         success = False
     return success
-
